@@ -2,12 +2,10 @@
 """
 Vedirect Utilities Helper.
 
-Contain Exception trowed in Vedirect package.
-And SerialUtils class, who extends ve_utils.utils.UType class,
+Extends ve_utils.utils.UType class,
 adding some regexes test patterns.
 
- .. seealso: VedirectController
- .. raises: SettingInvalidException
+ .. seealso: SerialConnection
 """
 import re
 from ve_utils.utils import UType
@@ -20,42 +18,22 @@ __status__ = "Production"
 __version__ = "1.0.0"
 
 
-class SettingInvalidException(Exception):
-    """
-    Some data must match the expected value/type
-
-    .. seealso:: Settings
-    """
-    pass
-
-
-class InputReadException(Exception):
-    """
-    Input read fails
-
-    .. seealso:: Settings
-    """
-    pass
-
-
-class TimeoutException(Exception):
-    """
-    Serial Timeout
-
-    .. seealso:: Settings
-    """
-    pass
-
-
 class SerialUtils (UType):
+    """
+    Vedirect Utilities Helper.
 
+    Extends ve_utils.utils.UType class,
+    adding some regexes test patterns.
+
+     .. seealso: SerialConnection
+    """
     @staticmethod
     def is_key_pattern(data: str) -> bool:
         """ Test if is valid key pattern """
         return SerialUtils.is_str(data)\
             and SerialUtils.is_list_not_empty(
             re.compile(
-                r"(?=[a-zA-Z0-9_]{1,30}$)^([a-zA-Z0-9]+(?:_[a-zA-Z0-9]+)*)$"
+                r"(?=\w{1,30}$)^([a-zA-Z\d]+(?:_[a-zA-Z\d]+)*)$"
                 ).findall(data)
         )
     
@@ -65,16 +43,34 @@ class SerialUtils (UType):
         return SerialUtils.is_str(data)\
             and SerialUtils.is_list_not_empty(
             re.compile(
-                r"(?=[a-zA-Z0-9_#]{1,30}$)^([a-zA-Z0-9#]+(?:_[a-zA-Z0-9#]+)*)$"
+                r"(?=[a-zA-Z\d_#]{1,30}$)^([a-zA-Z\d#]+(?:_[a-zA-Z\d#]+)*)$"
                 ).findall(data)
         )
-    
+
+    @staticmethod
+    def is_virtual_serial_port_pattern(data: str) -> bool:
+        """ Test if is valid unix virtual serial port pattern """
+        return SerialUtils.is_str(data) and SerialUtils.is_list_not_empty(
+            re.compile(
+                r'^(vmodem\d{1,3})$'
+            ).findall(data)
+        )
+
+    @staticmethod
+    def is_serial_port_name_pattern(data: str) -> bool:
+        """ Test if is valid serial port name pattern """
+        return SerialUtils.is_str(data) and SerialUtils.is_list_not_empty(
+            re.compile(
+                r'^((?:tty(?:USB|ACM)|vmodem|COM)\d{1,3})$'
+            ).findall(data)
+        )
+
     @staticmethod
     def is_unix_serial_port_pattern(data: str) -> bool:
         """ Test if is valid unix serial port pattern """
         return SerialUtils.is_str(data) and SerialUtils.is_list_not_empty(
             re.compile(
-                r'^((?:/dev/|/tmp/)((?:(?:tty(?:USB|ACM))|(?:vmodem))(?:\d{1,5})))$'
+                r'^(/dev/(tty(?:USB|ACM)\d{1,3}))$'
                 ).findall(data)
         )
     
